package org.synthesis.gui;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.ispeech.SpeechSynthesis;
import org.ispeech.SpeechSynthesisEvent;
import org.ispeech.error.BusyException;
import org.ispeech.error.InvalidApiKeyException;
import org.ispeech.error.NoNetworkException;
import org.synthesis.socket.DlFromServlet;
import org.synthesis.socket.SynthesisClient;

import android.app.Activity;
import android.content.Context;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.EditText;
import android.widget.Toast;

import com.nuance.nmdp.speechkit.SpeechError;
import com.nuance.nmdp.speechkit.SpeechKit;
import com.nuance.nmdp.speechkit.Vocalizer;

/**
 * 
 *
 */
public class TTSActivity extends Activity {
	
	private int nr = 0;
	private String[] stopList = Busstops.busStops;

	private static final String TAG = "SynthesisTest";
	SpeechSynthesis synthesis;
	Context _context;
	Long after, before, time; 

	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);		
		_context = this.getApplicationContext();
	
		setContentView(R.layout.tts);

		((EditText) findViewById(R.id.text)).setText(R.string.tts_sample_text);
		
		//iSpeech
		findViewById(R.id.speak).setOnClickListener(new OnSpeakListener());
		findViewById(R.id.stop).setOnClickListener(new OnStopListener());
		
		//Nora
		findViewById(R.id.nora).setOnClickListener(new OnNoraListener());
		
		//NoraServlet
		findViewById(R.id.noraservlet).setOnClickListener(new OnNoraServletListener());
		
		//Nuance
		findViewById(R.id.nuance).setOnClickListener(new OnNuanceListener());
		
		//Next
		findViewById(R.id.next).setOnClickListener(new OnNextListener());
		
		prepareTTSEngine();
	}


	private void prepareTTSEngine() {
		try {
			synthesis = SpeechSynthesis.getInstance(this);
			synthesis.setSpeechSynthesisEvent(new SpeechSynthesisEvent() {

				public void onPlaySuccessful() {
					Log.i(TAG, "onPlaySuccessful");
				}

				public void onPlayStopped() {
					Log.i(TAG, "onPlayStopped");
				}

				public void onPlayFailed(Exception e) {
					Log.e(TAG, "onPlayFailed");
					e.printStackTrace();
				}

				public void onPlayStart() {
					after = System.currentTimeMillis();
					Long time = after - before;
					Log.i(TAG, "iSpeech used "+time+" milliseconds.");
					Toast.makeText(_context, ""+time, Toast.LENGTH_SHORT).show();
					Log.i(TAG, "onPlayStart");
				}

				@Override
				public void onPlayCanceled() {
					Log.i(TAG, "onPlayCanceled");
				}
				
				
			});

			synthesis.setVoiceType("eurnorwegianfemale"); // All the values available to you can be found in the developer portal under your account

		} catch (InvalidApiKeyException e) {
			Log.e(TAG, "Invalid API key\n" + e.getStackTrace());
			Toast.makeText(_context, "ERROR: Invalid API key", Toast.LENGTH_LONG).show();
		}

	}


	private class OnSpeakListener implements OnClickListener {

		public void onClick(View v) {

			try {
				String ttsText = ((EditText) findViewById(R.id.text)).getText().toString();
				
				before = System.currentTimeMillis();
				
				synthesis.speak(ttsText);

			} catch (BusyException e) {
				Log.e(TAG, "SDK is busy");
				e.printStackTrace();
				Toast.makeText(_context, "ERROR: SDK is busy", Toast.LENGTH_LONG).show();
			} catch (NoNetworkException e) {
				Log.e(TAG, "Network is not available\n" + e.getStackTrace());
				Toast.makeText(_context, "ERROR: Network is not available", Toast.LENGTH_LONG).show();
			}
		}
	}
	
	//Nora
	private class OnNoraListener implements OnClickListener {

		public void onClick(View v) {

			try {
				String ttsText = ((EditText) findViewById(R.id.text)).getText().toString();
				
				before = System.currentTimeMillis();
				
				//Put file on SD-card
				// DELETE FILE FIRST. FIX!
				File sdCard = Environment.getExternalStorageDirectory();
				File dir = new File (sdCard.getAbsolutePath() + "/nora");
				if(!dir.exists())dir.mkdirs();
				File file = new File(dir, "nora_alaw.wav");
				
				SynthesisClient sc = new SynthesisClient();
			
				if(!sc.run(ttsText, file)){
					Toast.makeText(_context, "Connection Error", Toast.LENGTH_SHORT).show();
					return;
				}				
				
				after = System.currentTimeMillis();
				time = after - before;
				Log.i(TAG, "Nora used "+time+" milliseconds.");
				Toast.makeText(_context, ""+time, Toast.LENGTH_SHORT).show();
				
				playAudio("/sdcard/nora/","nora_alaw.wav");

			} catch (Exception e) {				
				e.printStackTrace();
				Log.e(TAG, "ERROR: "+e);
				//Toast.makeText(_context, "ERROR: SDK is busy", Toast.LENGTH_LONG).show();
			} 
		}
	}

	//NoraServlet
	private class OnNoraServletListener implements OnClickListener {

		public void onClick(View v) {

			try {
				String ttsText = ((EditText) findViewById(R.id.text)).getText().toString();
				
				before = System.currentTimeMillis();
				
				//Put file on SD-card
				// DELETE FILE FIRST. FIX!
				File sdCard = Environment.getExternalStorageDirectory();
				File dir = new File (sdCard.getAbsolutePath() + "/nora");
				if(!dir.exists())dir.mkdirs();
				File file = new File(dir, "nora_alaw.wav");
				
				if(!DlFromServlet.dl(ttsText, file)){
					Toast.makeText(_context, "Connection Error", Toast.LENGTH_SHORT).show();
					return;
				}				
				
				after = System.currentTimeMillis();
				time = after - before;
				Log.i(TAG, "NoraServlet used "+time+" milliseconds.");
				Toast.makeText(_context, ""+time, Toast.LENGTH_SHORT).show();
				
				playAudio("/sdcard/nora/","nora_alaw.wav");

			} catch (Exception e) {				
				e.printStackTrace();
				Log.e(TAG, "ERROR: "+e);
				//Toast.makeText(_context, "ERROR: SDK is busy", Toast.LENGTH_LONG).show();
			} 
		}
	}
	
	//Nuance
	private class OnNuanceListener implements OnClickListener {

		public void onClick(View v) {

			try {
				String ttsText = ((EditText) findViewById(R.id.text)).getText().toString();
				
				before = System.currentTimeMillis();
				

				SpeechKit speechKit = SpeechKit.initialize(getApplication().getApplicationContext(), AppInfo.SpeechKitAppId, AppInfo.SpeechKitServer, AppInfo.SpeechKitPort, AppInfo.SpeechKitSsl, AppInfo.SpeechKitApplicationKey);
	            speechKit.connect();
	            
	            // Create Vocalizer listener
	            Vocalizer.Listener vocalizerListener = new Vocalizer.Listener()
	            {
	                @Override
	                public void onSpeakingBegin(Vocalizer vocalizer, String text, Object context) {
	                	after = System.currentTimeMillis();
	    				time = after - before;
	    				Log.i(TAG, "Nuance used "+time+" milliseconds.");
	    				Toast.makeText(_context, ""+time, Toast.LENGTH_SHORT).show();
	                }

	                @Override
	                public void onSpeakingDone(Vocalizer vocalizer,
	                        String text, SpeechError error, Object context) 
	                {
	                   
	                }
	            };
				
				Vocalizer vocalizer = speechKit.createVocalizerWithLanguage("no_NO", vocalizerListener, new Handler());        
	            vocalizer.setVoice("Stine");
				
	            Object lastTtsContext = new Object();
                vocalizer.speakString(ttsText, lastTtsContext);

			} catch (Exception e) {				
				e.printStackTrace();
				Log.e(TAG, "ERROR: "+e);
				//Toast.makeText(_context, "ERROR: SDK is busy", Toast.LENGTH_LONG).show();
			} 
		}
	}
	
	//Next
	private class OnNextListener implements OnClickListener {
		public void onClick(View v) {
			if(nr == 1345)
				nr = 0;
			else
				nr++;
			((EditText) findViewById(R.id.text)).setText(stopList[nr]);
		}
	}
	
	
	public class OnStopListener implements OnClickListener {
		public void onClick(View v) {
			if (synthesis != null) {
				synthesis.stop();
			}
		}
	}



	@Override
	protected void onPause() {
		synthesis.stop();	//Optional to stop the playback when the activity is paused
		super.onPause();
	}

	public void playAudio(String path, String fileName){
	    //set up MediaPlayer    
	    MediaPlayer mp = new MediaPlayer();
	 
	    try {
	        mp.setDataSource(path+"/"+fileName);
	    } catch (IllegalArgumentException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
	    } catch (IllegalStateException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
	    } catch (IOException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
	    }
	    try {
	        mp.prepare();
	    } catch (IllegalStateException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
	    } catch (IOException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
	    }
	    mp.start();
	}


}