package org.synthesis.gui;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.ispeech.SpeechSynthesis;
import org.ispeech.SpeechSynthesisEvent;
import org.ispeech.error.BusyException;
import org.ispeech.error.InvalidApiKeyException;
import org.ispeech.error.NoNetworkException;
import org.synthesis.socket.DlFromServlet;
import org.synthesis.socket.SynthesisClient;

import android.app.Activity;
import android.content.Context;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.EditText;
import android.widget.Toast;

import com.nuance.nmdp.speechkit.SpeechError;
import com.nuance.nmdp.speechkit.SpeechKit;
import com.nuance.nmdp.speechkit.Vocalizer;

/**
 * 
 *
 */
public class TestActivity extends Activity {
	
	private String ttsText;
	private int nr = -1;
	private int module = -1;
	private static final int ISPEECH = 0;
	private static final int NORA = 1;
	private static final int NUANCE = 2;
	private static final int NORA_SERVLET = 3;
	
	
	private String[] stopList = Busstops.busStops;

	private static final String TAG = "SynthesisTest";
	SpeechSynthesis synthesis;
	Context _context;

	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);		
		_context = this.getApplicationContext();
	
		setContentView(R.layout.test);

		((EditText) findViewById(R.id.text)).setText("");
		
		//Random
		findViewById(R.id.random).setOnClickListener(new OnRandomListener());
		
		//Info
		findViewById(R.id.info).setOnClickListener(new OnInfoListener());

		//Repeat
		findViewById(R.id.repeat).setOnClickListener(new OnRepeatListener());
		
		//Dragvoll
		findViewById(R.id.dragvoll).setOnClickListener(new OnDragvollListener());
		
		
		
		prepareTTSEngine();
	}

	
	//Random
	private class OnRandomListener implements OnClickListener {
		public void onClick(View v) {
			//Reset
			((EditText) findViewById(R.id.text)).setText("");
			
			//Random bus stop
			nr = (int)(Math.random()*stopList.length);
			Log.e(TAG, "NR: "+nr);
			
			//Text
			ttsText = stopList[nr];

			module = (int)(Math.random()*3);
			switch(module){
				case  ISPEECH : 		runISpeech();
										break;
				case  NORA : 			runNora();
										break;
				case  NUANCE : 			runNuance();
										break;	
				case  NORA_SERVLET : 	runNoraServlet();
										break;									
			}
			
			Log.e(TAG, "module: "+module);
						
			
		}
	}
	
	//Info
	private class OnInfoListener implements OnClickListener {
		public void onClick(View v) {
			//Reset
			((EditText) findViewById(R.id.text)).setText("");
			
			String module_s="";
			
			switch(module){
				case  ISPEECH : 		module_s = "iSpeech";
										break;
				case  NORA : 			module_s = "Nora";
										break;
				case  NUANCE : 			module_s = "Nuance";
										break;	
				case  NORA_SERVLET : 	module_s = "Nora Servlet";
										break;								
			}
			
			((EditText) findViewById(R.id.text)).setText("Text: "+ttsText+"\nModule: "+module_s);
			
		}
	}
	
	//Repeat
	private class OnRepeatListener implements OnClickListener {
		public void onClick(View v) {
			switch(module){
				case  ISPEECH : 		runISpeech();
										break;
				case  NORA : 			runNora();
										break;
				case  NUANCE : 			runNuance();
										break;	
				case  NORA_SERVLET : 	runNoraServlet();
										break;									
			}
		}
	}
	
	//Dragvoll
	private class OnDragvollListener implements OnClickListener {
		public void onClick(View v) {
			ttsText = "Hva hres best ut?.. dragvoll.. Eller.. Dragvoll..";
			runNora();
		}
	}

	
	//Prepares iSpeech engine
	private void prepareTTSEngine() {
		try {
			synthesis = SpeechSynthesis.getInstance(this);
			synthesis.setSpeechSynthesisEvent(new SpeechSynthesisEvent() {

				public void onPlaySuccessful() {
					Log.i(TAG, "onPlaySuccessful");
				}

				public void onPlayStopped() {
					Log.i(TAG, "onPlayStopped");
				}

				public void onPlayFailed(Exception e) {
					Log.e(TAG, "onPlayFailed");
					e.printStackTrace();
				}

				public void onPlayStart() {
					Log.i(TAG, "onPlayStart");
				}

				@Override
				public void onPlayCanceled() {
					Log.i(TAG, "onPlayCanceled");
				}
				
				
			});

			synthesis.setVoiceType("eurnorwegianfemale"); // All the values available to you can be found in the developer portal under your account

		} catch (InvalidApiKeyException e) {
			Log.e(TAG, "Invalid API key\n" + e.getStackTrace());
			Toast.makeText(_context, "ERROR: Invalid API key", Toast.LENGTH_LONG).show();
		}

	}

	//iSpeech
	public void runISpeech(){
		try {			
			synthesis.speak(ttsText);

		} catch (BusyException e) {
			Log.e(TAG, "SDK is busy");
			e.printStackTrace();
			Toast.makeText(_context, "ERROR: SDK is busy", Toast.LENGTH_LONG).show();
		} catch (NoNetworkException e) {
			Log.e(TAG, "Network is not available\n" + e.getStackTrace());
			Toast.makeText(_context, "ERROR: Network is not available", Toast.LENGTH_LONG).show();
		}
	}
	
	//Nora
	public void runNora(){
		try {			
			//Put file on SD-card
			// DELETE FILE FIRST. FIX!
			File sdCard = Environment.getExternalStorageDirectory();
			File dir = new File (sdCard.getAbsolutePath() + "/nora");
			if(!dir.exists())dir.mkdirs();
			File file = new File(dir, "nora_alaw.wav");
			
			SynthesisClient sc = new SynthesisClient();
			sc.run(ttsText, file);
			playAudio("/sdcard/nora/","nora_alaw.wav");

		} catch (Exception e) {				
			e.printStackTrace();
			Log.e(TAG, "ERROR: "+e);
			//Toast.makeText(_context, "ERROR: SDK is busy", Toast.LENGTH_LONG).show();
		} 
	}

	//NoraServlet
	public void runNoraServlet(){
		try {
			//Put file on SD-card
			// DELETE FILE FIRST. FIX!
			File sdCard = Environment.getExternalStorageDirectory();
			File dir = new File (sdCard.getAbsolutePath() + "/nora");
			if(!dir.exists())dir.mkdirs();
			File file = new File(dir, "nora_alaw.wav");
			
			DlFromServlet.dl(ttsText, file);			
			playAudio("/sdcard/nora/","nora_alaw.wav");

		} catch (Exception e) {				
			e.printStackTrace();
			Log.e(TAG, "ERROR: "+e);
		} 
	}
	
	//Nuance
	public void runNuance(){
		try {
			SpeechKit speechKit = SpeechKit.initialize(getApplication().getApplicationContext(), AppInfo.SpeechKitAppId, AppInfo.SpeechKitServer, AppInfo.SpeechKitPort, AppInfo.SpeechKitSsl, AppInfo.SpeechKitApplicationKey);
            speechKit.connect();
            
            // Create Vocalizer listener
            Vocalizer.Listener vocalizerListener = new Vocalizer.Listener()
            {
                @Override
                public void onSpeakingBegin(Vocalizer vocalizer, String text, Object context) {

                }

                @Override
                public void onSpeakingDone(Vocalizer vocalizer,
                        String text, SpeechError error, Object context) 
                {
                   
                }
            };
			
			Vocalizer vocalizer = speechKit.createVocalizerWithLanguage("no_NO", vocalizerListener, new Handler());        
            vocalizer.setVoice("Stine");
			
            Object lastTtsContext = new Object();
            vocalizer.speakString(ttsText, lastTtsContext);

		} catch (Exception e) {				
			e.printStackTrace();
			Log.e(TAG, "ERROR: "+e);
			//Toast.makeText(_context, "ERROR: SDK is busy", Toast.LENGTH_LONG).show();
		} 
	}	
	
	public class OnStopListener implements OnClickListener {
		public void onClick(View v) {
			if (synthesis != null) {
				synthesis.stop();
			}
		}
	}



	@Override
	protected void onPause() {
		synthesis.stop();	//Optional to stop the playback when the activity is paused
		super.onPause();
	}

	public void playAudio(String path, String fileName){
	    //set up MediaPlayer    
	    MediaPlayer mp = new MediaPlayer();
	 
	    try {
	        mp.setDataSource(path+"/"+fileName);
	    } catch (IllegalArgumentException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
	    } catch (IllegalStateException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
	    } catch (IOException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
	    }
	    try {
	        mp.prepare();
	    } catch (IllegalStateException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
	    } catch (IOException e) {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
	    }
	    mp.start();
	}


}
