package org.synthesis.socket;



import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.io.RandomAccessFile;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.net.SocketException;

import android.util.Log;

/**
 *
 * @author Trond
 */
public class SynthesisClient {
    
    Socket requestSocket = null;
    BufferedReader in;
    RandomAccessFile randomAccessWriter;
    byte[] buffer;
    private static final String TAG = "SynthesisTest";

    public void setupSocket()
    {
        try
        {
            // Connect to server, with specified timeout
            InetAddress addr = InetAddress.getByName("orwell.idi.ntnu.no");
            int port = 8888;
            SocketAddress sockaddr = new InetSocketAddress(addr, port);
            requestSocket = new Socket();
            requestSocket.connect(sockaddr, 10000);
        } catch (SocketException e)
        {
            Log.i(TAG,"SOCKET FAIL");
            e.printStackTrace();
        } catch (IOException e)
        {
            Log.i(TAG,"IO FAIL");
            e.printStackTrace();
        }
    }

    public boolean run(String input, File path)
    {
        boolean check = false;
        try
        {
            setupSocket();
            Log.i(TAG,"Connected");
            
            // Send a test query
            BufferedWriter bufferedWriter = new BufferedWriter(new OutputStreamWriter(requestSocket.getOutputStream(), "ISO8859-1"));
            bufferedWriter.write(input);
            bufferedWriter.flush();

            // Receive input
            InputStream is = requestSocket.getInputStream();

            //Read from inputstream
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            int nRead;
            byte[] data = new byte[16384];
            while ((nRead = is.read(data, 0, data.length)) != -1) {
                baos.write(data, 0, nRead);
            }
            baos.flush();           
            buffer = baos.toByteArray();
            is.close();
            bufferedWriter.close();
            
            
            //-- A-law --
            Log.i(TAG,"Creating WAVE header for the A-law file...");
            
			FileOutputStream fos = new FileOutputStream(path);          
            //Header
            short numOfChannels_alaw = 1;
            short bitsPerSample_alaw = 8;
            WaveHeader alawHeader = new WaveHeader(WaveHeader.FORMAT_ALAW, numOfChannels_alaw, 8000, bitsPerSample_alaw, buffer.length);
            alawHeader.write(fos);            
            fos.write(buffer); //Data
            fos.close();
            Log.i(TAG,"Done.");            

            check = true;
        } catch (Exception e)
        {
            e.printStackTrace();
            Log.i(TAG,"Error: "+e);
            
            //  System.exit(1);
        } finally{
            try
            {
                if(requestSocket != null)
                {
                    requestSocket.close();
                    Log.i(TAG,"Socket closed: " + requestSocket.isClosed());
                    Log.i(TAG,"Returning " + check);
                    return check;
                }
            } catch (Exception e)
            {
                Log.i(TAG,"Exception closing socket");
            }

        }
        return check;
    }
}
